@@ -13,8 +13,11 @@ CREATE TEMP TABLE temp_history_uint (
 
 DO $$
 DECLARE
-	chunk_tm_interval	INTEGER;
 	jobid			INTEGER;
+	tsdb_version		TEXT;
+	tsdb_version_major	INTEGER;
+	tsdb_version_minor	INTEGER;
+	compress_after		INTEGER;
 BEGIN
 	PERFORM create_hypertable('history_uint', 'clock', chunk_time_interval => (
 		SELECT integer_interval FROM timescaledb_information.dimensions WHERE hypertable_name='history_uint_old'
