@@ -19,12 +19,12 @@
 #define ZBX_STR(str)	ZBX_STR2(str)
 
 #define APPLICATION_NAME	"Zabbix Agent"
-#define ZABBIX_REVDATE		"30 July 2025"
+#define ZABBIX_REVDATE		"25 August 2025"
 #define ZABBIX_VERSION_MAJOR	7
 #define ZABBIX_VERSION_MINOR	4
-#define ZABBIX_VERSION_PATCH	1
+#define ZABBIX_VERSION_PATCH	2
 #ifndef ZABBIX_VERSION_REVISION
-#	define ZABBIX_VERSION_REVISION	439ef6b1409
+#	define ZABBIX_VERSION_REVISION	7aa4e0782fe
 #endif
 #ifdef _WINDOWS
 #	ifndef ZABBIX_VERSION_RC_NUM
