@@ -1,5 +1,5 @@
 /*
-** Copyright (C) 2001-2026 Zabbix SIA
+** Copyright (C) 2001-2025 Zabbix SIA
 **
 ** This program is free software: you can redistribute it and/or modify it under the terms of
 ** the GNU Affero General Public License as published by the Free Software Foundation, version 3.
@@ -22,7 +22,7 @@
 typedef int (*zbx_rtc_notify_generic_cb_t)(zbx_ipc_async_socket_t *rtc, unsigned char process_type, int process_num,
 		zbx_uint32_t code, const char *data, zbx_uint32_t size);
 
-void	zbx_init_escalations(int escalators_num, zbx_rtc_notify_generic_cb_t rtc_notify_cb);
+void	zbx_init_escalations(int escalators_num, zbx_rtc_notify_generic_cb_t rtc_notify_generic_func);
 void	zbx_start_escalations(zbx_ipc_async_socket_t *rtc, zbx_vector_escalation_new_ptr_t *escalations);
 void	zbx_escalation_new_ptr_free(zbx_escalation_new_t *data);
 
