@@ -250,7 +250,7 @@ func run() error {
 			return errs.Wrap(err, "cannot send remote command")
 		}
 
-		fmt.Fprintf(os.Stderr, "%s\n", reply)
+		fmt.Fprintf(os.Stdout, "%s\n", reply)
 
 		return nil
 	}
