@@ -22,13 +22,13 @@ import (
 )
 
 const (
-	ZABBIX_REVDATE          = "30 July 2025"
+	ZABBIX_REVDATE          = "25 August 2025"
 	ZABBIX_VERSION_MAJOR    = 7
 	ZABBIX_VERSION_MINOR    = 4
-	ZABBIX_VERSION_PATCH    = 1
+	ZABBIX_VERSION_PATCH    = 2
 	ZABBIX_VERSION_RC       = ""
 	ZABBIX_VERSION_RC_NUM   = "1400"
-	ZABBIX_VERSION_REVISION = "439ef6b1409"
+	ZABBIX_VERSION_REVISION = "7aa4e0782fe"
 	copyrightMessage        = "Copyright (C) 2025 Zabbix SIA\n" +
 		"License AGPLv3: GNU Affero General Public License version 3 <https://www.gnu.org/licenses/>.\n" +
 		"This is free software: you are free to change and redistribute it according to\n" +
