@@ -1,5 +1,5 @@
 /*
-** Copyright (C) 2001-2026 Zabbix SIA
+** Copyright (C) 2001-2025 Zabbix SIA
 **
 ** This program is free software: you can redistribute it and/or modify it under the terms of
 ** the GNU Affero General Public License as published by the Free Software Foundation, version 3.
@@ -20,7 +20,6 @@ import (
 
 	"github.com/go-ole/go-ole"
 	"github.com/go-ole/go-ole/oleutil"
-	"golang.zabbix.com/sdk/errs"
 	"golang.zabbix.com/sdk/log"
 )
 
@@ -96,85 +95,74 @@ func isPropertyKeyProperty(propsCol *ole.IDispatch) (isKeyProperty bool, err err
 func (r *valueResult) write(rs *ole.IDispatch) (err error) {
 	v, err := oleutil.GetProperty(rs, "Count")
 	if err != nil {
-		return errs.Wrap(err, "failed to get row count")
+		return err
 	}
 	defer clearOle(v)
-
 	if v.Val == 0 {
-		return errs.New("empty WMI search result")
+		return errors.New("Empty WMI search result.")
 	}
 
 	var propertyKeyFieldValue any
 
-	oleErr := oleutil.ForEach(rs, func(vr *ole.VARIANT) error {
+	oleErr := oleutil.ForEach(rs, func(vr *ole.VARIANT) (err error) {
 		row := vr.ToIDispatch()
 		defer row.Release()
 
-		rawProps, rowErr := row.GetProperty("Properties_")
-		if rowErr != nil {
-			return errs.Wrap(rowErr, "failed to get item property row")
+		rawProps, err := row.GetProperty("Properties_")
+		if err != nil {
+			return
 		}
 
 		props := rawProps.ToIDispatch()
 		defer props.Release()
 
-		rowErr = oleutil.ForEach(props, func(vc *ole.VARIANT) error {
+		err = oleutil.ForEach(props, func(vc *ole.VARIANT) (err error) {
 			propsCol := vc.ToIDispatch()
 			defer propsCol.Release()
 
-			isKeyProperty, colErr := isPropertyKeyProperty(propsCol)
-			if colErr != nil {
-				return colErr
-			}
-
-			propsName, colErr := oleutil.GetProperty(propsCol, "Name")
-			if colErr != nil {
-				return errs.Wrap(colErr, "failed to get item name")
+			propsName, err := oleutil.GetProperty(propsCol, "Name")
+			if err != nil {
+				return
 			}
 			defer clearOle(propsName)
 
-			propsVal, colErr := oleutil.GetProperty(propsCol, "Value")
-			if colErr != nil {
-				return errs.Wrap(colErr, "failed to get item value")
+			propsVal, err := oleutil.GetProperty(propsCol, "Value")
+			if err != nil {
+				return
 			}
 			defer clearOle(propsVal)
 
-			if isKeyProperty {
-				// remember key field value in the case it was the only selected column
-				propertyKeyFieldValue = propsVal.Value()
-
-				return nil
-			}
-
 			if propsVal.Value() == nil {
-				return errs.New("empty WMI search result")
+				return errors.New("Empty WMI search result.")
 			}
 
-			r.data = propsVal.Value()
+			isKeyProperty, err := isPropertyKeyProperty(propsCol)
+			if err != nil {
+				return
+			}
 
-			return stopErrorCol
+			if !isKeyProperty {
+				r.data = propsVal.Value()
+				return stopErrorCol
+			}
+			// remember key field value in the case it was the only selected column
+			propertyKeyFieldValue = propsVal.Value()
+			return
 		})
-		if rowErr == nil {
+
+		if err == nil {
 			return stopErrorRow
 		}
-
-		return rowErr //nolint:wrapcheck // keeping "empty WMI search result" unwrapped
+		return
 	})
-
-	stop, ok := oleErr.(stopError) //nolint:errorlint // stopError type does not work with errors.As()
-	if !ok {
-		return oleErr //nolint:wrapcheck // keeping "empty WMI search result" unwrapped
-	}
-
-	if oleErr == nil || stop == stopErrorRow {
-		if propertyKeyFieldValue == nil {
-			return errs.New("empty WMI search result")
+	if stop, ok := oleErr.(stopError); !ok {
+		return oleErr
+	} else {
+		if oleErr == nil || stop == stopErrorRow {
+			r.data = propertyKeyFieldValue
 		}
-
-		r.data = propertyKeyFieldValue
 	}
-
-	return nil
+	return
 }
 
 type tableResult struct {
