@@ -453,7 +453,17 @@ func ExchangeWithRedirect(addrpool AddressSet, localAddr *net.Addr, timeout time
 retry:
 	retries++
 
-	b, errs, err := Exchange(addrpool, localAddr, timeout, connectTimeout, data, args...)
+	var exchangeAddrPool AddressSet
+
+	if retries > 1 {
+		// On retries, use a temporary addrpool with only the redirect address to skip failover
+		tempAddrs := []string{addrpool.Get()}
+		exchangeAddrPool = NewAddressPool(tempAddrs)
+	} else {
+		exchangeAddrPool = addrpool
+	}
+
+	b, errs, err := Exchange(exchangeAddrPool, localAddr, timeout, connectTimeout, data, args...)
 
 	if errs != nil {
 		return b, errs, err
