@@ -1,5 +1,5 @@
 /*
-** Copyright (C) 2001-2026 Zabbix SIA
+** Copyright (C) 2001-2025 Zabbix SIA
 **
 ** This program is free software: you can redistribute it and/or modify it under the terms of
 ** the GNU Affero General Public License as published by the Free Software Foundation, version 3.
@@ -15,12 +15,13 @@
 package oracle
 
 import (
+	"path/filepath"
+
 	"golang.zabbix.com/sdk/conf"
 	"golang.zabbix.com/sdk/errs"
 	"golang.zabbix.com/sdk/plugin"
 )
 
-// Session type contains session parameters of the config file.
 type Session struct {
 	// URI defines an address of the Oracle Net Listener.
 	URI string `conf:"name=Uri,optional"`
@@ -33,7 +34,7 @@ type Session struct {
 	Service string `conf:"optional"`
 }
 
-// PluginOptions option from the config file.
+// PluginOptions option from config file.
 type PluginOptions struct {
 	// ConnectTimeout is the maximum time in seconds for waiting when a connection has to be established.
 	// Default value equals to the global timeout.
@@ -52,8 +53,8 @@ type PluginOptions struct {
 	// CustomQueriesPath is a full pathname of a directory containing *.sql files with custom queries.
 	CustomQueriesPath string `conf:"optional"`
 
-	// ResolveTNS enables the interpretation of a connection string (ConnString) in a metrics key as TNS.
-	ResolveTNS bool `conf:"optional,default=false"`
+	// CustomQueriesEnabled enables custom query key.
+	CustomQueriesEnabled bool `conf:"optional,default=false"`
 
 	// Default stores default connection parameter values from configuration file
 	Default Session `conf:"optional"`
@@ -66,6 +67,8 @@ func (p *Plugin) Configure(global *plugin.GlobalOptions, options any) {
 		p.Errf("cannot unmarshal configuration options: %s", err)
 	}
 
+	p.options.setCustomQueriesPathDefault()
+
 	if p.options.ConnectTimeout == 0 {
 		p.options.ConnectTimeout = global.Timeout
 	}
@@ -77,12 +80,16 @@ func (p *Plugin) Configure(global *plugin.GlobalOptions, options any) {
 
 // Validate implements the Configurator interface.
 // Returns an error if validation of a plugin's configuration is failed.
-func (p *Plugin) Validate(options any) error { //nolint:revive
+func (p *Plugin) Validate(options interface{}) error {
 	var opts PluginOptions
 
 	err := conf.UnmarshalStrict(options, &opts)
 	if err != nil {
-		return errs.Wrap(err, "plugin config validation failed")
+		return errs.Wrap(err, "failed to unmarshal configuration options")
+	}
+
+	if opts.CustomQueriesEnabled && opts.CustomQueriesPath != "" && !filepath.IsAbs(opts.CustomQueriesPath) {
+		return errs.Errorf("opto.CustomQueriesPath path: '%s' must be absolute", opts.CustomQueriesPath)
 	}
 
 	return nil
