@@ -62,6 +62,7 @@ void	zbx_am_db_alert_free(zbx_am_db_alert_t *alert)
 void	zbx_am_media_clear(zbx_am_media_t *media)
 {
 	zbx_free(media->sendto);
+	zbx_free(media->mediatype_params);
 }
 
 void	zbx_am_media_free(zbx_am_media_t *media)
@@ -642,12 +643,14 @@ zbx_uint32_t	zbx_alerter_serialize_medias(unsigned char **data, zbx_am_media_t *
 
 	for (int i = 0; i < medias_num; i++)
 	{
-		zbx_uint32_t	data_len = 0, sendto_len;
+		zbx_uint32_t	data_len = 0, sendto_len, params_len;
 		zbx_am_media_t	*media = medias[i];
 
 		zbx_serialize_prepare_value(data_len, media->mediaid);
 		zbx_serialize_prepare_value(data_len, media->mediatypeid);
 		zbx_serialize_prepare_str_len(data_len, media->sendto, sendto_len);
+		zbx_serialize_prepare_value(data_len, media->mediatype_type);
+		zbx_serialize_prepare_str_len(data_len, media->mediatype_params, params_len);
 
 		while (data_len > data_alloc - data_offset)
 		{
@@ -657,7 +660,9 @@ zbx_uint32_t	zbx_alerter_serialize_medias(unsigned char **data, zbx_am_media_t *
 		ptr = *data + data_offset;
 		ptr += zbx_serialize_value(ptr, media->mediaid);
 		ptr += zbx_serialize_value(ptr, media->mediatypeid);
-		(void)zbx_serialize_str(ptr, media->sendto, sendto_len);
+		ptr += zbx_serialize_str(ptr, media->sendto, sendto_len);
+		ptr += zbx_serialize_value(ptr, media->mediatype_type);
+		(void)zbx_serialize_str(ptr, media->mediatype_params, params_len);
 
 		data_offset += data_len;
 	}
@@ -687,6 +692,8 @@ void	zbx_alerter_deserialize_medias(const unsigned char *data, zbx_am_media_t **
 		data += zbx_deserialize_value(data, &media->mediaid);
 		data += zbx_deserialize_value(data, &media->mediatypeid);
 		data += zbx_deserialize_str(data, &media->sendto, len);
+		data += zbx_deserialize_value(data, &media->mediatype_type);
+		data += zbx_deserialize_str(data, &media->mediatype_params, len);
 
 		(*medias)[i] = media;
 	}
