@@ -68,6 +68,13 @@ zbx_audit_entry_t	*zbx_audit_entry_init(zbx_uint64_t id, const int id_table, con
 	return audit_entry;
 }
 
+void	zbx_audit_entry_clean(zbx_audit_entry_t *entry)
+{
+	zbx_json_free(&entry->details_json);
+	zbx_free(entry->name);
+	zbx_free(entry->cuid);
+}
+
 zbx_audit_entry_t	*zbx_audit_entry_init_cuid(const char *cuid, const int id_table, const char *name,
 		int audit_action, int resource_type)
 {
@@ -283,9 +290,7 @@ void	zbx_audit_clean(int audit_context_mode)
 
 	while (NULL != (audit_entry = (zbx_audit_entry_t **)zbx_hashset_iter_next(&iter)))
 	{
-		zbx_json_free(&((*audit_entry)->details_json));
-		zbx_free((*audit_entry)->name);
-		zbx_free((*audit_entry)->cuid);
+		zbx_audit_entry_clean(*audit_entry);
 		zbx_free(*audit_entry);
 	}
 
@@ -308,6 +313,7 @@ void	zbx_audit_prepare(int audit_context_mode)
 
 	zbx_config_get(&cfg, ZBX_CONFIG_FLAGS_AUDITLOG_ENABLED | ZBX_CONFIG_FLAGS_AUDITLOG_MODE);
 	zbx_audit_init(cfg.auditlog_enabled, cfg.auditlog_mode, audit_context_mode);
+	zbx_config_clean(&cfg);
 }
 
 static int	zbx_audit_validate_entry(const zbx_audit_entry_t *entry)
@@ -436,6 +442,7 @@ int	audit_field_value_matches_db_default(const char *table_name, const char *fie
 		if (NULL == (table = zbx_db_get_table(table_name)))
 		{
 			zabbix_log(LOG_LEVEL_CRIT, "%s(): cannot find table '%s'", __func__, table_name);
+			*cached_table_name = '\0';
 			THIS_SHOULD_NEVER_HAPPEN;
 			return FAIL;
 		}
@@ -665,7 +672,7 @@ void	zbx_audit_update_json_delete(const zbx_uint64_t id, const int id_table, con
 	delete_json(&((*found_audit_entry)->details_json), audit_op, key);
 }
 
-zbx_audit_entry_t	*zbx_audit_get_entry(zbx_uint64_t id, const char *cuid, int id_table)
+zbx_audit_entry_t	*audit_get_entry(zbx_uint64_t id, const char *cuid, int id_table)
 {
 	zbx_audit_entry_t	local_audit_entry, *plocal_audit_entry = &local_audit_entry, **paudit_entry;
 
@@ -682,54 +689,282 @@ zbx_audit_entry_t	*zbx_audit_get_entry(zbx_uint64_t id, const char *cuid, int id
 	return *paudit_entry;
 }
 
-void	zbx_audit_entry_append_int(zbx_audit_entry_t *entry, int audit_op, const char *key, ...)
+/******************************************************************************
+ *                                                                            *
+ * Purpose: create key for audit entry based on resource type and name        *
+ *                                                                            *
+ * Parameters: entry    - [IN] audit entry                                    *
+ *             name     - [IN] name to be included in the key                 *
+ *             key      - [OUT] resulting key                                 *
+ *             key_size - [IN] size of key buffer                             *
+ *                                                                            *
+ ******************************************************************************/
+static void	audit_entry_make_key(const zbx_audit_entry_t *entry, const char *name, char *key, size_t key_size)
 {
-	va_list		args;
-	int		value1, value2;
+	const char	*owner;
 
-	va_start(args, key);
-	value1 = va_arg(args, int);
-
-	switch (audit_op)
+	switch (entry->resource_type)
 	{
-		case ZBX_AUDIT_ACTION_ADD:
-			append_int_json(&entry->details_json, AUDIT_DETAILS_ACTION_ADD, key, value1);
+		case ZBX_AUDIT_RESOURCE_HOST:
+			owner = "host";
 			break;
-		case ZBX_AUDIT_ACTION_UPDATE:
-			value2 = va_arg(args, int);
-			update_int_json(&entry->details_json, key, value1, value2);
+		case ZBX_AUDIT_RESOURCE_GRAPH:
+			owner = "graph";
+			break;
+		case ZBX_AUDIT_RESOURCE_TRIGGER:
+			owner = "trigger";
+			break;
+		case ZBX_AUDIT_RESOURCE_HOST_GROUP:
+			owner = "hostgroup";
+			break;
+		case ZBX_AUDIT_RESOURCE_ITEM:
+			owner = "item";
+			break;
+		case ZBX_AUDIT_RESOURCE_SCENARIO:
+			owner = "scenario";
+			break;
+		case ZBX_AUDIT_RESOURCE_SCRIPT:
+			owner = "script";
+			break;
+		case ZBX_AUDIT_RESOURCE_PROXY:
+			owner = "proxy";
+			break;
+		case ZBX_AUDIT_RESOURCE_TRIGGER_PROTOTYPE:
+			owner = "triggerprototype";
+			break;
+		case ZBX_AUDIT_RESOURCE_GRAPH_PROTOTYPE:
+			owner = "graphprototype";
+			break;
+		case ZBX_AUDIT_RESOURCE_ITEM_PROTOTYPE:
+			owner = "itemprototype";
+			break;
+		case ZBX_AUDIT_RESOURCE_HOST_PROTOTYPE:
+			owner = "hostprototype";
+			break;
+		case ZBX_AUDIT_RESOURCE_SETTINGS:
+			owner = "settings";
+			break;
+		case ZBX_AUDIT_RESOURCE_HA_NODE:
+			owner = "hanode";
+			break;
+		case ZBX_AUDIT_RESOURCE_LLD_RULE:
+			owner = "discoveryrule";
+			break;
+		case ZBX_AUDIT_RESOURCE_HISTORY:
+			owner = "history";
+			break;
+		case ZBX_AUDIT_RESOURCE_LLD_RULE_PROTOTYPE:
+			owner = "discoveryruleprototype";
+			break;
+		case ZBX_AUDIT_RESOURCE_MEDIATYPE:
+			owner = "mediatype";
 			break;
 		default:
-			THIS_SHOULD_NEVER_HAPPEN;
-			break;
+			THIS_SHOULD_NEVER_HAPPEN_MSG("Unknown resource type: %d", entry->resource_type);
+			exit(EXIT_FAILURE);
 	}
 
-	va_end(args);
+	zbx_snprintf(key, key_size, "%s.%s", owner, name);
 }
 
-void	zbx_audit_entry_append_string(zbx_audit_entry_t *entry, int audit_op, const char *key, ...)
+/******************************************************************************
+ *                                                                            *
+ * Purpose: add detail creation for audit entry                               *
+ *                                                                            *
+ ******************************************************************************/
+void	zbx_audit_entry_add(zbx_audit_entry_t *entry, const char *name)
 {
-	va_list		args;
-	const char	*value1, *value2;
+	char	key[AUDIT_DETAILS_KEY_LEN];
 
-	va_start(args, key);
-	value1 = va_arg(args, const char *);
+	/* auditlog has been disabled */
+	if (NULL == entry)
+		return;
 
-	switch (audit_op)
+	audit_entry_make_key(entry, name, key, sizeof(key));
+	append_json_no_value(&entry->details_json, AUDIT_DETAILS_ACTION_ADD, key);
+}
+
+/******************************************************************************
+ *                                                                            *
+ * Purpose: add detail update for audit entry                                 *
+ *                                                                            *
+ ******************************************************************************/
+void	zbx_audit_entry_update(zbx_audit_entry_t *entry, const char *name)
+{
+	char	key[AUDIT_DETAILS_KEY_LEN];
+
+	/* auditlog has been disabled */
+	if (NULL == entry)
+		return;
+
+	audit_entry_make_key(entry, name, key, sizeof(key));
+	append_json_no_value(&entry->details_json, AUDIT_DETAILS_ACTION_UPDATE, key);
+}
+
+void	zbx_audit_entry_delete(zbx_audit_entry_t *entry, const char *name)
+{
+	char	key[AUDIT_DETAILS_KEY_LEN];
+
+	/* auditlog has been disabled */
+	if (NULL == entry)
+		return;
+
+	audit_entry_make_key(entry, name, key, sizeof(key));
+	delete_json(&entry->details_json, AUDIT_DETAILS_ACTION_DELETE, key);
+}
+
+
+/******************************************************************************
+ *                                                                            *
+ * Purpose: add integer key creation for audit entry                          *
+ *                                                                            *
+ ******************************************************************************/
+void	zbx_audit_entry_add_int(zbx_audit_entry_t *entry, const char *table, const char *field, const char *name,
+		int value1)
+{
+	char	key[AUDIT_DETAILS_KEY_LEN];
+
+	/* auditlog has been disabled */
+	if (NULL == entry)
+		return;
+
+	audit_entry_make_key(entry, name, key, sizeof(key));
+
+	if (NULL != table)
 	{
-		case ZBX_AUDIT_ACTION_ADD:
-			append_str_json(&entry->details_json, AUDIT_DETAILS_ACTION_ADD, key, value1);
-			break;
-		case ZBX_AUDIT_ACTION_UPDATE:
-			value2 = va_arg(args, const char *);
-			update_str_json(&entry->details_json, key, value1, value2);
-			break;
-		default:
-			THIS_SHOULD_NEVER_HAPPEN;
-			break;
+		char	buffer[MAX_ID_LEN];
+
+		zbx_snprintf(buffer, sizeof(buffer), "%d", value1);
+		if (SUCCEED == audit_field_value_matches_db_default(table, field, buffer, 0))
+			return;
 	}
 
-	va_end(args);
+	append_int_json(&entry->details_json, AUDIT_DETAILS_ACTION_ADD, key, value1);
+}
+
+/******************************************************************************
+ *                                                                            *
+ * Purpose: add integer key update for audit entry                            *
+ *                                                                            *
+ ******************************************************************************/
+void	zbx_audit_entry_update_int(zbx_audit_entry_t *entry, const char *name, int value1, int value2)
+{
+	char	key[AUDIT_DETAILS_KEY_LEN];
+
+	/* auditlog has been disabled */
+	if (NULL == entry)
+		return;
+
+	audit_entry_make_key(entry, name, key, sizeof(key));
+	update_int_json(&entry->details_json, key, value1, value2);
+}
+
+/******************************************************************************
+ *                                                                            *
+ * Purpose: add uint64 key creation for audit entry                           *
+ *                                                                            *
+ ******************************************************************************/
+void	zbx_audit_entry_add_uint64(zbx_audit_entry_t *entry, const char *table, const char *field, const char *name,
+		zbx_uint64_t value1)
+{
+	char	key[AUDIT_DETAILS_KEY_LEN];
+
+	/* auditlog has been disabled */
+	if (NULL == entry)
+		return;
+
+	audit_entry_make_key(entry, name, key, sizeof(key));
+
+	if (NULL != table)
+	{
+		char	buffer[MAX_ID_LEN];
+
+		zbx_snprintf(buffer, sizeof(buffer), ZBX_FS_UI64, value1);
+		if (SUCCEED == audit_field_value_matches_db_default(table, field, buffer, value1))
+			return;
+	}
+
+	append_int_json(&entry->details_json, AUDIT_DETAILS_ACTION_ADD, key, value1);
+}
+
+/******************************************************************************
+ *                                                                            *
+ * Purpose: add uint64 key update for audit entry                             *
+ *                                                                            *
+ ******************************************************************************/
+void	zbx_audit_entry_update_uint64(zbx_audit_entry_t *entry, const char *name, zbx_uint64_t value1,
+		zbx_uint64_t value2)
+{
+	char	key[AUDIT_DETAILS_KEY_LEN];
+
+	/* auditlog has been disabled */
+	if (NULL == entry)
+		return;
+
+	audit_entry_make_key(entry, name, key, sizeof(key));
+	update_uint64_json(&entry->details_json, key, value1, value2);
+}
+
+/******************************************************************************
+ *                                                                            *
+ * Purpose: add string key creation for audit entry                           *
+ *                                                                            *
+ ******************************************************************************/
+void	zbx_audit_entry_add_string(zbx_audit_entry_t *entry, const char *table, const char *field, const char *name,
+		const char *value1)
+{
+	char	key[AUDIT_DETAILS_KEY_LEN];
+
+	/* auditlog has been disabled */
+	if (NULL == entry)
+		return;
+
+	audit_entry_make_key(entry, name, key, sizeof(key));
+
+	if (NULL != table && SUCCEED == audit_field_value_matches_db_default(table, field, value1, 0))
+		return;
+
+	append_str_json(&entry->details_json, AUDIT_DETAILS_ACTION_ADD, key, value1);
+}
+
+/******************************************************************************
+ *                                                                            *
+ * Purpose: add string key update for audit entry                             *
+ *                                                                            *
+ ******************************************************************************/
+void	zbx_audit_entry_update_string(zbx_audit_entry_t *entry, const char *name, const char *value1,
+		const char *value2)
+{
+	char	key[AUDIT_DETAILS_KEY_LEN];
+
+	/* auditlog has been disabled */
+	if (NULL == entry)
+		return;
+
+	audit_entry_make_key(entry, name, key, sizeof(key));
+	update_str_json(&entry->details_json, key, value1, value2);
+}
+
+/******************************************************************************
+ *                                                                            *
+ * Purpose: add secret key creation for audit entry                           *
+ *                                                                            *
+ ******************************************************************************/
+void	zbx_audit_entry_add_secret(zbx_audit_entry_t *entry, const char *table, const char *field, const char *name,
+		const char *value1)
+{
+	char	key[AUDIT_DETAILS_KEY_LEN];
+
+	/* auditlog has been disabled */
+	if (NULL == entry)
+		return;
+
+	audit_entry_make_key(entry, name, key, sizeof(key));
+
+	if (NULL != table && SUCCEED == audit_field_value_matches_db_default(table, field, value1, 0))
+		return;
+
+	append_str_json(&entry->details_json, AUDIT_DETAILS_ACTION_ADD, key, ZBX_MACRO_SECRET_MASK);
 }
 
 /******************************************************************************
@@ -777,7 +1012,23 @@ int	zbx_auditlog_history_push(zbx_uint64_t userid, const char *username, const c
 	zbx_db_insert_clean(&db_insert);
 	zbx_json_free(&details_json);
 out:
+	zbx_config_clean(&cfg);
+
 	zabbix_log(LOG_LEVEL_TRACE, "End of %s():%s", __func__, zbx_result_string(ret));
 
 	return ret;
 }
+
+/******************************************************************************
+ *                                                                            *
+ * Purpose: get audit status                                                  *
+ *                                                                            *
+ * Parameters: audit_context_mode - [IN] audit context mode                   *
+ *             enabled            - [OUT] audit status (1 - enabled)          *
+ *                                                                            *
+ ******************************************************************************/
+void	zbx_audit_get_status(int audit_context_mode, int *enabled)
+{
+	RETURN_IF_AUDIT_OFF(audit_context_mode);
+	*enabled = 1;
+}
