@@ -170,7 +170,7 @@ void	zbx_dc_config_history_sync_get_items_by_itemids(zbx_history_sync_item_t *it
 	size_t			i;
 	const ZBX_DC_ITEM	*dc_item;
 	const ZBX_DC_HOST	*dc_host = NULL;
-	zbx_config_hk_t		config_hk;
+	zbx_config_hk_t		config_hk = {0};
 	zbx_dc_um_handle_t	*um_handle;
 	zbx_dc_config_t		*dc_config = get_dc_config();
 
@@ -560,6 +560,8 @@ static void	dc_get_history_recv_item(zbx_history_recv_item_t *dst_item, const ZB
 	const ZBX_DC_TRAPITEM	*trapitem;
 	const ZBX_DC_HTTPITEM	*httpitem;
 
+	dst_item->preprocessing = zbx_dc_item_requires_preprocessing(src_item);
+
 	dst_item->type = src_item->type;
 	dst_item->value_type = src_item->value_type;
 	dst_item->state = ITEM_STATE_NORMAL;
