@@ -12,7 +12,15 @@
 ** If not, see <https://www.gnu.org/licenses/>.
 **/
 
+#include <fcntl.h>
+#include <stdlib.h>
+
+#if defined(_WINDOWS) || defined(__MINGW32__)
+#	include <stddef.h>
+#endif
+
 #include "zbxcommon.h"
+#include "zbxtypes.h"
 
 #if defined(_WINDOWS) || defined(__MINGW32__)
 #	include "zbxstr.h"
@@ -81,27 +89,34 @@ const char	*get_program_name(const char *path)
  ******************************************************************************/
 void	*zbx_calloc2(const char *filename, int line, void *old, size_t nmemb, size_t size)
 {
-	int	max_attempts;
 	void	*ptr = NULL;
 
 	/* old pointer must be NULL */
 	if (NULL != old)
 	{
-		zabbix_log(LOG_LEVEL_CRIT, "[file:%s,line:%d] zbx_calloc: allocating already allocated memory. "
+		zabbix_log(LOG_LEVEL_CRIT,
+				"[file:%s,line:%d] zbx_calloc: allocating already allocated memory. "
 				"Please report this to Zabbix developers.",
 				filename, line);
 	}
 
-	for (
-		max_attempts = 10, nmemb = MAX(nmemb, 1), size = MAX(size, 1);
-		0 < max_attempts && NULL == ptr;
-		ptr = calloc(nmemb, size), max_attempts--
-	);
+	if (0 == nmemb || 0 == size)
+	{
+		zabbix_log(LOG_LEVEL_DEBUG,
+				"[file:%s,line:%d] zbx_calloc: "
+				"allocating " ZBX_FS_SIZE_T " memory objects of size " ZBX_FS_SIZE_T ". "
+				"Please report this to Zabbix developers.",
+				filename, line, (zbx_fs_size_t)nmemb, (zbx_fs_size_t)size);
+	}
+	nmemb = MAX(nmemb, 1);
+	size = MAX(size, 1);
 
+	ptr = calloc(nmemb, size);
 	if (NULL != ptr)
 		return ptr;
 
-	zabbix_log(LOG_LEVEL_CRIT, "[file:%s,line:%d] zbx_calloc: out of memory. Requested " ZBX_FS_SIZE_T " bytes.",
+	zabbix_log(LOG_LEVEL_CRIT,
+			"[file:%s,line:%d] zbx_calloc: out of memory. Requested " ZBX_FS_SIZE_T " bytes.",
 			filename, line, (zbx_fs_size_t)size);
 
 	exit(EXIT_FAILURE);
@@ -116,27 +131,34 @@ void	*zbx_calloc2(const char *filename, int line, void *old, size_t nmemb, size_
  ******************************************************************************/
 void	*zbx_malloc2(const char *filename, int line, void *old, size_t size)
 {
-	int	max_attempts;
 	void	*ptr = NULL;
 
 	/* old pointer must be NULL */
 	if (NULL != old)
 	{
-		zabbix_log(LOG_LEVEL_CRIT, "[file:%s,line:%d] zbx_malloc: allocating already allocated memory. "
+		zabbix_log(LOG_LEVEL_CRIT,
+				"[file:%s,line:%d] zbx_malloc: allocating already allocated memory. "
 				"Please report this to Zabbix developers.",
 				filename, line);
 	}
 
-	for (
-		max_attempts = 10, size = MAX(size, 1);
-		0 < max_attempts && NULL == ptr;
-		ptr = malloc(size), max_attempts--
-	);
+	if (0 == size)
+	{
+		zabbix_log(LOG_LEVEL_DEBUG,
+				"[file:%s,line:%d] zbx_malloc: "
+				"allocating memory object of size " ZBX_FS_SIZE_T " bytes. "
+				"Please report this to Zabbix developers.",
+				filename, line, (zbx_fs_size_t)size);
+	}
+	size = MAX(size, 1);
 
+	ptr = malloc(size);
 	if (NULL != ptr)
 		return ptr;
 
-	zabbix_log(LOG_LEVEL_CRIT, "[file:%s,line:%d] zbx_malloc: out of memory. Requested " ZBX_FS_SIZE_T " bytes.",
+	zabbix_log(LOG_LEVEL_CRIT,
+			"[file:%s,line:%d] zbx_malloc: out of memory. "
+			"Requested " ZBX_FS_SIZE_T " bytes.",
 			filename, line, (zbx_fs_size_t)size);
 
 	exit(EXIT_FAILURE);
@@ -152,19 +174,24 @@ void	*zbx_malloc2(const char *filename, int line, void *old, size_t size)
  ******************************************************************************/
 void	*zbx_realloc2(const char *filename, int line, void *old, size_t size)
 {
-	int	max_attempts;
 	void	*ptr = NULL;
 
-	for (
-		max_attempts = 10, size = MAX(size, 1);
-		0 < max_attempts && NULL == ptr;
-		ptr = realloc(old, size), max_attempts--
-	);
+	if (0 == size)
+	{
+		zabbix_log(LOG_LEVEL_DEBUG,
+				"[file:%s,line:%d] zbx_realloc: "
+				"allocating memory object of size " ZBX_FS_SIZE_T " bytes. "
+				"Please report this to Zabbix developers.",
+				filename, line, (zbx_fs_size_t)size);
+	}
+	size = MAX(size, 1);
 
+	ptr = realloc(old, size);
 	if (NULL != ptr)
 		return ptr;
 
-	zabbix_log(LOG_LEVEL_CRIT, "[file:%s,line:%d] zbx_realloc: out of memory. Requested " ZBX_FS_SIZE_T " bytes.",
+	zabbix_log(LOG_LEVEL_CRIT,
+			"[file:%s,line:%d] zbx_realloc: out of memory. Requested " ZBX_FS_SIZE_T " bytes.",
 			filename, line, (zbx_fs_size_t)size);
 
 	exit(EXIT_FAILURE);
@@ -172,18 +199,16 @@ void	*zbx_realloc2(const char *filename, int line, void *old, size_t size)
 
 char	*zbx_strdup2(const char *filename, int line, char *old, const char *str)
 {
-	int	retry;
 	char	*ptr = NULL;
 
 	zbx_free(old);
 
-	for (retry = 10; 0 < retry && NULL == ptr; ptr = strdup(str), retry--)
-		;
-
+	ptr = strdup(str);
 	if (NULL != ptr)
 		return ptr;
 
-	zabbix_log(LOG_LEVEL_CRIT, "[file:%s,line:%d] zbx_strdup: out of memory. Requested " ZBX_FS_SIZE_T " bytes.",
+	zabbix_log(LOG_LEVEL_CRIT,
+			"[file:%s,line:%d] zbx_strdup: out of memory. Requested " ZBX_FS_SIZE_T " bytes.",
 			filename, line, (zbx_fs_size_t)(strlen(str) + 1));
 
 	exit(EXIT_FAILURE);
