@@ -1,5 +1,5 @@
 /*
-** Copyright (C) 2001-2026 Zabbix SIA
+** Copyright (C) 2001-2025 Zabbix SIA
 **
 ** This program is free software: you can redistribute it and/or modify it under the terms of
 ** the GNU Affero General Public License as published by the Free Software Foundation, version 3.
@@ -12,24 +12,26 @@
 ** If not, see <https://www.gnu.org/licenses/>.
 **/
 
+#include "embed.h"
+#include "duktape.h"
+
+#include "zbxembed.h"
+
+#ifdef HAVE_LIBCURL
+
 #include "browser_alert.h"
 #include "browser_element.h"
 #include "browser_error.h"
 #include "browser_perf.h"
-#include "config.h"
-#include "duktape.h"
-#include "embed.h"
 #include "webdriver.h"
 #include "zbxalgo.h"
 #include "zbxcommon.h"
-#include "zbxembed.h"
+#include "zbxjson.h"
 #include "zbxtime.h"
 #include "zbxtypes.h"
 #include "zbxvariant.h"
 #include "zbxstr.h"
 
-#ifdef HAVE_LIBCURL
-
 /******************************************************************************
  *                                                                            *
  * Purpose: return backing C structure embedded in browser object         *
@@ -1383,12 +1385,12 @@ static int	es_browser_create_prototype(duk_context *ctx)
 	duk_push_c_function(ctx, es_browser_ctor, 1);
 	duk_push_object(ctx);
 
-	es_put_function_list(ctx, -1, browser_methods);
+	duk_put_function_list(ctx, -1, browser_methods);
 
 	if (1 != duk_put_prop_string(ctx, -2, "prototype"))
 		return FAIL;
 
-	es_put_function_list(ctx, -1, browser_static_methods);
+	duk_put_function_list(ctx, -1, browser_static_methods);
 
 	if (1 != duk_put_global_string(ctx, "Browser"))
 		return FAIL;
