@@ -1,5 +1,5 @@
 /*
-** Copyright (C) 2001-2026 Zabbix SIA
+** Copyright (C) 2001-2025 Zabbix SIA
 **
 ** This program is free software: you can redistribute it and/or modify it under the terms of
 ** the GNU Affero General Public License as published by the Free Software Foundation, version 3.
@@ -13,17 +13,18 @@
 **/
 
 #include "browser_element.h"
-#include "browser_error.h"
+
 #include "duktape.h"
-#include "embed.h"
-#include "webdriver.h"
-#include "zbxalgo.h"
-#include "zbxcommon.h"
-#include "zbxembed.h"
-#include "zbxtypes.h"
 
 #ifdef HAVE_LIBCURL
 
+#include "browser_error.h"
+#include "embed.h"
+#include "webdriver.h"
+
+#include "zbxembed.h"
+#include "zbxalgo.h"
+
 /******************************************************************************
  *                                                                            *
  * Purpose: return backing C structure embedded in element object             *
@@ -104,7 +105,8 @@ static duk_ret_t	wd_element_ctor(duk_context *ctx, zbx_webdriver_t *wd, const ch
 
 	duk_push_string(ctx, "browser");
 	duk_push_heapptr(ctx, wd->browser);
-	duk_def_prop(ctx, -3, ZBX_ES_DEFPROP_READONLY);
+	duk_def_prop(ctx, -3, DUK_DEFPROP_HAVE_VALUE | DUK_DEFPROP_CLEAR_WRITABLE | DUK_DEFPROP_HAVE_ENUMERABLE |
+			DUK_DEFPROP_HAVE_CONFIGURABLE);
 
 	duk_push_c_function(ctx, wd_element_dtor, 1);
 	duk_set_finalizer(ctx, -2);
@@ -373,7 +375,7 @@ static const duk_function_list_entry	element_methods[] = {
 void	wd_element_create(duk_context *ctx, zbx_webdriver_t *wd, const char *elementid)
 {
 	wd_element_ctor(ctx, wd, elementid);
-	es_put_function_list(ctx, -1, element_methods);
+	duk_put_function_list(ctx, -1, element_methods);
 }
 
 /******************************************************************************
