@@ -24,8 +24,6 @@
 
 #define ZBX_ES_LOG_MEMORY_LIMIT	(ZBX_MEBIBYTE * 8)
 #define ZBX_ES_LOG_MSG_LIMIT	8000
-#define ZBX_ES_DEFPROP_READONLY	(DUK_DEFPROP_HAVE_VALUE | DUK_DEFPROP_CLEAR_WRITABLE |	\
-			DUK_DEFPROP_CLEAR_CONFIGURABLE | DUK_DEFPROP_CLEAR_ENUMERABLE)
 
 /* this macro can be used in time intensive C functions to check for script timeout execution */
 #define ZBX_ES_CHECK_TIMEOUT(ctx, env)									\
@@ -90,6 +88,5 @@ zbx_es_obj_type_t;
 void	es_obj_attach_data(zbx_es_env_t *env, void *objptr, void *data, zbx_es_obj_type_t type);
 void	*es_obj_get_data(zbx_es_env_t *env, const void *objptr, zbx_es_obj_type_t type);
 void	*es_obj_detach_data(zbx_es_env_t *env, void *objptr, zbx_es_obj_type_t type);
-void	es_put_function_list(duk_context *ctx, duk_idx_t obj_idx, const duk_function_list_entry *funcs);
 
 #endif /* ZABBIX_EMBED_H */
