@@ -1,5 +1,5 @@
 /*
-** Copyright (C) 2001-2026 Zabbix SIA
+** Copyright (C) 2001-2025 Zabbix SIA
 **
 ** This program is free software: you can redistribute it and/or modify it under the terms of
 ** the GNU Affero General Public License as published by the Free Software Foundation, version 3.
@@ -15,8 +15,9 @@
 #ifndef ZABBIX_PP_CACHE_H
 #define ZABBIX_PP_CACHE_H
 
-#include "zbxpreproc.h"
 #include "zbxvariant.h"
+#include "zbxpreprocbase.h"
+#include "zbxjson.h"
 
 typedef struct
 {
@@ -35,7 +36,6 @@ typedef struct
 }
 zbx_pp_cache_t;
 
-int	pp_cache_get_type(int step_type);
 zbx_pp_cache_t	*pp_cache_create(const zbx_pp_item_preproc_t *preproc, const zbx_variant_t *value);
 void		pp_cache_release(zbx_pp_cache_t *cache);
 zbx_pp_cache_t	*pp_cache_copy(zbx_pp_cache_t *cache);
