@@ -1,27 +1,20 @@
 /*
-** Zabbix
 ** Copyright (C) 2001-2025 Zabbix SIA
 **
-** This program is free software; you can redistribute it and/or modify
-** it under the terms of the GNU General Public License as published by
-** the Free Software Foundation; either version 2 of the License, or
-** (at your option) any later version.
+** This program is free software: you can redistribute it and/or modify it under the terms of
+** the GNU Affero General Public License as published by the Free Software Foundation, version 3.
 **
-** This program is distributed in the hope that it will be useful,
-** but WITHOUT ANY WARRANTY; without even the implied warranty of
-** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-** GNU General Public License for more details.
+** This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
+** without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+** See the GNU Affero General Public License for more details.
 **
-** You should have received a copy of the GNU General Public License
-** along with this program; if not, write to the Free Software
-** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+** You should have received a copy of the GNU Affero General Public License along with this program.
+** If not, see <https://www.gnu.org/licenses/>.
 **/
 
-#include "common.h"
-#include "sysinfo.h"
-#include "log.h"
+#include "../sysinfo.h"
 
-int	SYSTEM_SWAP_SIZE(AGENT_REQUEST *request, AGENT_RESULT *result)
+int	system_swap_size(AGENT_REQUEST *request, AGENT_RESULT *result)
 {
 	struct sysinfo	info;
 	char		*swapdev, *mode;
@@ -65,7 +58,8 @@ int	SYSTEM_SWAP_SIZE(AGENT_REQUEST *request, AGENT_RESULT *result)
 	}
 	else if (0 == strcmp(mode, "pused"))
 	{
-		SET_DBL_RESULT(result, 0 != info.totalswap ? 100.0 - 100.0 * (info.freeswap / (double)info.totalswap) : 0.0);
+		SET_DBL_RESULT(result, 0 != info.totalswap ? 100.0 - 100.0 * (info.freeswap / (double)info.totalswap) :
+				0.0);
 	}
 	else
 	{
@@ -94,8 +88,8 @@ swap_stat_t;
 		if (6 != sscanf(line, "%d %d %*d %*s "					\
 				ZBX_FS_UI64 " %*d " ZBX_FS_UI64 " %*d "			\
 				ZBX_FS_UI64 " %*d " ZBX_FS_UI64 " %*d %*d %*d %*d",	\
-				&rdev_major, 		/* major */			\
-				&rdev_minor, 		/* minor */			\
+				&rdev_major,		/* major */			\
+				&rdev_minor,		/* minor */			\
 				&result->rio,		/* rio */			\
 				&result->rsect,		/* rsect */			\
 				&result->wio,		/* wio */			\
@@ -108,8 +102,8 @@ swap_stat_t;
 		if (6 != sscanf(line, "%u %u %*s "					\
 				ZBX_FS_UI64 " %*d " ZBX_FS_UI64 " %*d "			\
 				ZBX_FS_UI64 " %*d " ZBX_FS_UI64 " %*d %*d %*d %*d",	\
-				&rdev_major, 		/* major */			\
-				&rdev_minor, 		/* minor */			\
+				&rdev_major,		/* major */			\
+				&rdev_minor,		/* minor */			\
 				&result->rio,		/* rio */			\
 				&result->rsect,		/* rsect */			\
 				&result->wio,		/* wio */			\
@@ -118,8 +112,8 @@ swap_stat_t;
 			if (6 != sscanf(line, "%u %u %*s "				\
 					ZBX_FS_UI64 " " ZBX_FS_UI64 " "			\
 					ZBX_FS_UI64 " " ZBX_FS_UI64,			\
-					&rdev_major, 		/* major */		\
-					&rdev_minor, 		/* minor */		\
+					&rdev_major,		/* major */		\
+					&rdev_minor,		/* minor */		\
 					&result->rio,		/* rio */		\
 					&result->rsect,		/* rsect */		\
 					&result->wio,		/* wio */		\
@@ -197,7 +191,7 @@ static int	get_swap_pages(swap_stat_t *result)
 #endif
 			ret = SYSINFO_RET_OK;
 			break;
-		};
+		}
 
 		zbx_fclose(f);
 	}
@@ -259,7 +253,7 @@ static int	get_swap_stat(const char *swapdev, swap_stat_t *result)
 	return ret;
 }
 
-int	SYSTEM_SWAP_IN(AGENT_REQUEST *request, AGENT_RESULT *result)
+int	system_swap_in(AGENT_REQUEST *request, AGENT_RESULT *result)
 {
 	char		*swapdev, *mode;
 	swap_stat_t	ss;
@@ -302,7 +296,7 @@ int	SYSTEM_SWAP_IN(AGENT_REQUEST *request, AGENT_RESULT *result)
 	return SYSINFO_RET_OK;
 }
 
-int	SYSTEM_SWAP_OUT(AGENT_REQUEST *request, AGENT_RESULT *result)
+int	system_swap_out(AGENT_REQUEST *request, AGENT_RESULT *result)
 {
 	char		*swapdev, *mode;
 	swap_stat_t	ss;
