@@ -25,10 +25,10 @@
 #include "zbxtime.h"
 #include "zbxnum.h"
 
-int	zbx_hashicorp_kvs_get(const char *vault_url, const char *prefix, const char *token, const char *ssl_cert_file,
-		const char *ssl_key_file, const char *config_source_ip, const char *config_ssl_ca_location,
-		const char *config_ssl_cert_location, const char *config_ssl_key_location, const char *path,
-		long timeout, zbx_kvs_t *kvs, char **error)
+int	zbx_vault_get_kvs_hashicorp(const char *vault_url, const char *prefix, const char *token,
+		const char *ssl_cert_file, const char *ssl_key_file, const char *config_source_ip,
+		const char *config_ssl_ca_location, const char *config_ssl_cert_location,
+		const char *config_ssl_key_location, const char *path, long timeout, zbx_kvs_t *kvs, char **error)
 {
 #ifndef HAVE_LIBCURL
 	ZBX_UNUSED(vault_url);
@@ -122,7 +122,7 @@ fail:
 #endif
 }
 
-void	zbx_hashicorp_renew_token(const char *vault_url, const char *token, const char *ssl_cert_file,
+void	zbx_vault_renew_token_hashicorp(const char *vault_url, const char *token, const char *ssl_cert_file,
 		const char *ssl_key_file, const char *config_source_ip, const char *config_ssl_ca_location,
 		const char *config_ssl_cert_location, const char *config_ssl_key_location, long timeout)
 {
