@@ -233,7 +233,7 @@ static unsigned char	get_host_monitored_by(zbx_uint64_t src_proxyid, zbx_uint64_
  * Return value: hostid - new/existing hostid                                 *
  *                                                                            *
  ******************************************************************************/
-static zbx_uint64_t	add_discovered_host(const zbx_db_event *event, int *status, zbx_config_t *cfg)
+static zbx_uint64_t	add_discovered_host(const zbx_db_event *event, int *status, const zbx_config_t *cfg)
 {
 	zbx_db_result_t		result, result2;
 	zbx_db_row_t		row, row2;
@@ -330,7 +330,7 @@ static zbx_uint64_t	add_discovered_host(const zbx_db_event *event, int *status,
 						" where h.hostid=i.hostid"
 							" and i.ip=ds.ip"
 							" and h.status in (%d,%d)"
-							" and h.flags<>%d"
+							" and h.flags&%d=0"
 							" and h.monitored_by=%u"
 							" and ds.dhostid=" ZBX_FS_UI64,
 							HOST_STATUS_MONITORED, HOST_STATUS_NOT_MONITORED,
@@ -606,7 +606,7 @@ static zbx_uint64_t	add_discovered_host(const zbx_db_event *event, int *status,
 					"select hostid,proxyid,name,status,proxy_groupid,monitored_by"
 					" from hosts"
 					" where host='%s'"
-						" and flags<>%d"
+						" and flags&%d=0"
 						" and status in (%d,%d)"
 					" order by hostid",
 					host_esc, ZBX_FLAG_DISCOVERY_PROTOTYPE,
@@ -766,7 +766,6 @@ out:
 		zbx_db_free_result(result);
 	}
 clean:
-	zbx_config_clean(cfg);
 	zbx_vector_uint64_destroy(&groupids);
 	zbx_free(hostname);
 
@@ -973,7 +972,7 @@ static void	discovered_host_tags_save(zbx_uint64_t hostid, zbx_vector_db_tag_ptr
  *             cfg   - [IN] global configuration data                         *
  *                                                                            *
  ******************************************************************************/
-void	op_host_add(const zbx_db_event *event, zbx_config_t *cfg)
+void	op_host_add(const zbx_db_event *event, const zbx_config_t *cfg)
 {
 	int	status;
 
@@ -1014,7 +1013,7 @@ void	op_host_del(const zbx_db_event *event)
 	zbx_vector_str_create(&hostnames);
 	zbx_vector_str_append(&hostnames, zbx_strdup(NULL, hostname));
 
-	zbx_db_delete_hosts_with_prototypes(&hostids, &hostnames, zbx_map_db_event_to_audit_context(event));
+	zbx_db_delete_hosts(&hostids, &hostnames, zbx_map_db_event_to_audit_context(event));
 	hostname_esc = zbx_db_dyn_escape_string(hostname);
 	zbx_db_execute("delete from autoreg_host where host='%s'", hostname_esc);
 
@@ -1283,7 +1282,7 @@ out:
  *             lnk_templateids - [IN] array of template IDs                   *
  *                                                                            *
  ******************************************************************************/
-void	op_template_add(const zbx_db_event *event, zbx_config_t *cfg, zbx_vector_uint64_t *lnk_templateids)
+void	op_template_add(const zbx_db_event *event, const zbx_config_t *cfg, zbx_vector_uint64_t *lnk_templateids)
 {
 	zbx_uint64_t	hostid;
 	char		*error = NULL;
