@@ -574,6 +574,8 @@ class CControllerPopupGeneric extends CController {
 			'real_hosts' =>							'in 1',
 			'normal_only' =>						'in 1',
 			'with_graphs' =>						'in 1',
+			'with_hosts' =>							'in 1',
+			'with_templates' =>						'in 1',
 			'with_graph_prototypes' =>				'in 1',
 			'with_items' =>							'in 1',
 			'with_simple_graph_items' =>			'in 1',
@@ -1368,6 +1370,10 @@ class CControllerPopupGeneric extends CController {
 					$options['with_httptests'] = true;
 				}
 
+				if ($this->hasInput('with_hosts')) {
+					$options['with_hosts'] = true;
+				}
+
 				if ($this->hasInput('with_items')) {
 					$options['with_items'] = true;
 				}
@@ -1407,6 +1413,10 @@ class CControllerPopupGeneric extends CController {
 					$options['with_items'] = true;
 				}
 
+				if ($this->hasInput('with_templates')) {
+					$options['with_templates'] = true;
+				}
+
 				$records = API::TemplateGroup()->get($options);
 				if ($this->hasInput('enrich_parent_groups')) {
 					$records = enrichParentTemplateGroups($records);
