@@ -1,6 +1,6 @@
 <?php
 /*
-** Copyright (C) 2001-2026 Zabbix SIA
+** Copyright (C) 2001-2025 Zabbix SIA
 **
 ** This program is free software: you can redistribute it and/or modify it under the terms of
 ** the GNU Affero General Public License as published by the Free Software Foundation, version 3.
@@ -43,7 +43,7 @@ class CRadioButtonList extends CList {
 	 */
 	protected $values = [];
 
-	public function __construct($name, $value) {
+	public function __construct($name, $value = null) {
 		parent::__construct();
 
 		$this->name = $name;
@@ -54,6 +54,8 @@ class CRadioButtonList extends CList {
 		$this->modern = false;
 		$this->readonly = false;
 		$this->setId(zbx_formatDomId($name));
+
+		$this->setAttribute('data-field-type', 'radio-list');
 	}
 
 	public function setValues(array $values): self {
