@@ -305,40 +305,6 @@ function hex2rgb($color) {
 	return [hexdec($r), hexdec($g), hexdec($b)];
 }
 
-function getColorVariations($color, $variations_requested = 1) {
-	if ($variations_requested <= 1) {
-		return [$color];
-	}
-
-	$change = hex2rgb('#ffffff'); // Color which is increased/decreased in variations.
-	$max = 50;
-
-	$color = hex2rgb($color);
-	$variations = [];
-
-	$range = range(-1 * $max, $max, $max * 2 / $variations_requested);
-
-	// Remove redundant values.
-	while (count($range) > $variations_requested) {
-		(count($range) % 2) ? array_shift($range) : array_pop($range);
-	}
-
-	// Calculate colors.
-	foreach ($range as $var) {
-		$r = $color[0] + ($change[0] / 100 * $var);
-		$g = $color[1] + ($change[1] / 100 * $var);
-		$b = $color[2] + ($change[2] / 100 * $var);
-
-		$variations[] = '#' . rgb2hex([
-			$r < 0 ? 0 : ($r > 255 ? 255 : (int) $r),
-			$g < 0 ? 0 : ($g > 255 ? 255 : (int) $g),
-			$b < 0 ? 0 : ($b > 255 ? 255 : (int) $b)
-		]);
-	}
-
-	return $variations;
-}
-
 /**
  * Convert suffixed string to decimal bytes ('10K' => 10240).
  * Note: this function must not depend on optional PHP libraries, since it is used in Zabbix setup.
@@ -1243,18 +1209,6 @@ function zbx_objectValues($value, $field) {
 	return $result;
 }
 
-function zbx_cleanHashes(&$value) {
-	if (is_array($value)) {
-		// reset() is needed to move internal array pointer to the beginning of the array
-		reset($value);
-		if (zbx_ctype_digit(key($value))) {
-			$value = array_values($value);
-		}
-	}
-
-	return $value;
-}
-
 function zbx_toCSV($values) {
 	$csv = '';
 	$glue = '","';
@@ -1642,19 +1596,20 @@ function detect_page_type($default = PAGE_TYPE_HTML) {
  *                                            ZBX_STYLE_MSG_GOOD, ZBX_STYLE_MSG_BAD, ZBX_STYLE_MSG_WARNING.
  * @param array       $messages               An array of messages.
  * @param string      $messages[]['message']  Message text.
- * @param string|null $title                  (optional) Message box title.
+ * @param mixed       $title                  (optional) Message box title.
  * @param bool        $show_close_box         (optional) Show or hide close button in error message box.
  * @param bool        $show_details           (optional) Show or hide message details.
  *
  * @return CTag
  */
-function makeMessageBox(string $class, array $messages, ?string $title = null, bool $show_close_box = true,
+function makeMessageBox(string $class, array $messages, mixed $title = null, bool $show_close_box = true,
 		bool $show_details = false): CTag {
 
 	$aria_labels = [
 		ZBX_STYLE_MSG_GOOD => _('Success message'),
-		ZBX_STYLE_MSG_BAD => _('Error message'),
-		ZBX_STYLE_MSG_WARNING => _('Warning message')
+		ZBX_STYLE_MSG_INFO => _('Info message'),
+		ZBX_STYLE_MSG_WARNING => _('Warning message'),
+		ZBX_STYLE_MSG_BAD => _('Error message')
 	];
 
 	$message_box = (new CTag('output', true))
