@@ -1,5 +1,5 @@
 /*
-** Copyright (C) 2001-2026 Zabbix SIA
+** Copyright (C) 2001-2025 Zabbix SIA
 **
 ** This program is free software: you can redistribute it and/or modify it under the terms of
 ** the GNU Affero General Public License as published by the Free Software Foundation, version 3.
@@ -19,7 +19,7 @@
 (function($) {
 	'use strict';
 
-	function enableHandle() {
+	function enableHandle(_, on_activate = false) {
 		var $btn_change = $(this),
 			$input = $btn_change.siblings('input[type=password]'),
 			$btn_undo = $btn_change
@@ -29,8 +29,15 @@
 		$input
 			.prop('disabled', false)
 			.attr('placeholder', t('value'))
-			.val('')
-			.focus();
+			.val('');
+
+		if (!on_activate) {
+			$input[0].dispatchEvent(new Event('input', {bubbles: true}));
+
+			setTimeout(() => {
+				$input.focus();
+			});
+		}
 
 		$btn_change.prop('disabled', true);
 		$btn_undo.show();
@@ -45,6 +52,11 @@
 					.off('click', enableHandle)
 					.on('click', enableHandle);
 			});
+		},
+		activateInput() {
+			return this.each(function() {
+				enableHandle.call($('.btn-change', $(this)), null, true);
+			});
 		}
 	};
 
