@@ -69,7 +69,8 @@
 				data.push({
 					id: id,
 					name: item.name,
-					prefix: (typeof item.prefix === 'undefined') ? '' : item.prefix
+					prefix: item.prefix === undefined ? '' : item.prefix,
+					isNew: item.isNew !== undefined && item.isNew
 				});
 			}
 
@@ -249,8 +250,11 @@
 					 *      Note: hidden and disabled items will not submit to the server.
 					 *   3. The "change" trigger must fire.
 					 */
+					const new_item_pattern = ms.options.newItemName
+						? `input[name="${ms.options.newItemName}"]`
+						: 'input[name*="[new]"]';
 
-					$('input[name*="[new]"]', $obj)
+					$(new_item_pattern, $obj)
 						.prop('disabled', !ms.options['addNew'])
 						.each(function() {
 							var id = $(this).val();
@@ -460,6 +464,7 @@
 
 			options.required_str = $obj.attr('aria-required') === undefined ? 'false' : $obj.attr('aria-required');
 			$obj.removeAttr('aria-required');
+			$obj.attr('data-field-type', 'multiselect');
 
 			var ms = {
 				options: options,
@@ -971,9 +976,11 @@
 		var prefix = (item.prefix || ''),
 			item_disabled = (typeof item.disabled !== 'undefined' && item.disabled);
 
+		const new_item_name = ms.options.newItemName ?? ms.options.name + '[new]';
+
 		$obj.append($('<input>', {
 			type: 'hidden',
-			name: (ms.options.addNew && item.isNew) ? ms.options.name + '[new]' : ms.options.name,
+			name: (ms.options.addNew && item.isNew) ? new_item_name : ms.options.name,
 			value: item.id,
 			'data-name': item.name,
 			'data-prefix': prefix
@@ -1426,7 +1433,7 @@
 	function updateSearchFieldVisibility($obj) {
 		var ms = $obj.data('multiSelect'),
 			visible_now = !$obj.hasClass('search-disabled'),
-			visible = !ms.options.disabled
+			visible = !ms.options.disabled && !ms.options.readonly
 				&& (ms.options.selectedLimit == 0 || $('.selected li', $obj).length < ms.options.selectedLimit);
 
 		if (visible === visible_now) {
