@@ -38,10 +38,34 @@ $available_js = [
 	'class.widget.inaccessible.js' => '',
 	'class.widget.iterator.js' => '',
 	'class.widget.misconfigured.js' => '',
-	'class.widget.paste-placeholder.js' => '',
+	'class.widget.create-placeholder.js' => '',
+	'class.widget-field.js' => '',
+	'class.widget-edit.dialogue.js' => '',
+	'class.widget-edit.sandbox.js' => '',
+	'class.widget-edit.validator.js' => '',
+	'class.widget-field.checkbox.js' => '',
 	'class.widget-field.checkbox-list.js' => '',
+	'class.widget-field.color.js' => '',
+	'class.widget-field.date-picker.js' => '',
+	'class.widget-field.integer-box.js' => '',
+	'class.widget-field.lat-lng.js' => '',
 	'class.widget-field.multiselect.js' => '',
+	'class.widget-field.numeric-box.js' => '',
+	'class.widget-field.pattern-select.js' => '',
+	'class.widget-field.radio-button-list.js' => '',
+	'class.widget-field.range-control.js' => '',
+	'class.widget-field.select.js' => '',
+	'class.widget-field.severities.js' => '',
+	'class.widget-field.tags.js' => '',
+	'class.widget-field.text-area.js' => '',
+	'class.widget-field.text-box.js' => '',
+	'class.widget-field.thresholds.js' => '',
 	'class.widget-field.time-period.js' => '',
+	'class.widget-field.url.js' => '',
+	'class.widget-field.time-zone.js' => '',
+	'class.widget-field-event.js' => '',
+	'class.widget-form.js' => '',
+	'class.widget-form-event.js' => '',
 	'class.widget-select.popup.js' => '',
 	'hostinterfacemanager.js' => '',
 	'hostmacrosmanager.js' => '',
@@ -54,7 +78,6 @@ $available_js = [
 	'flickerfreescreen.js' => '',
 	'multilineinput.js' => '',
 	'multiselect.js' => '',
-	'colorpicker.js' => '',
 	'chkbxrange.js' => '',
 	'layout.mode.js' => '',
 	'textareaflexible.js' => '',
@@ -69,17 +92,19 @@ $available_js = [
 	'qrcode.js' => 'vendors/qrcode/',
 	// classes
 	'component.z-bar-gauge.js' => '',
-	'component.z-vertical.js' => '',
+	'component.z-color-picker.js' => '',
 	'component.z-select.js' => '',
 	'component.z-sparkline.js' => '',
+	'component.z-vertical.js' => '',
 	'class.event-hub.js' => '',
 	'class.event-hub.event.js' => '',
 	'class.base-component.js' => '',
 	'class.calendar.js' => '',
 	'class.cdate.js' => '',
 	'class.cdebug.js' => '',
-	'class.cmap.js' => '',
 	'class.expandable.subfilter.js' => '',
+	'class.form.js' => '',
+	'class.form.validator.js' => '',
 	'class.geomaps.js' => '',
 	'class.localstorage.js' => '',
 	'class.menu.js' => '',
@@ -105,8 +130,14 @@ $available_js = [
 	'class.sidebar.js' => '',
 	'class.software-version-check.js' => '',
 	'class.sortable.js' => '',
-	'class.svg.canvas.js' => 'vector/',
-	'class.svg.map.js' => 'vector/',
+	'class.imagecache.js' => '',
+	'class.svgcanvas.js' => 'vector/canvas/',
+	'class.svgtextarea.js' => 'vector/canvas/',
+	'class.svgelement.js' => 'vector/canvas/',
+	'class.svgmap.js' => 'vector/svgmap/',
+	'class.svgmapelement.js' => 'vector/svgmap/',
+	'class.svgmaplink.js' => 'vector/svgmap/',
+	'class.svgmapshape.js' => 'vector/svgmap/',
 	'class.cviewswitcher.js' => '',
 	'class.rpc.js' => '',
 	'class.tabfilter.js' => '',
@@ -121,9 +152,31 @@ $available_js = [
 	// templates
 	'sysmap.tpl.js' => 'templates/',
 	// page-specific scripts
+	'class.linkform.js' => 'pages/sysmap/forms/',
+	'class.massshapeform.js' => 'pages/sysmap/forms/',
+	'class.shapeform.js' => 'pages/sysmap/forms/',
+	'class.massform.js' => 'pages/sysmap/forms/',
+	'class.selementform.js' => 'pages/sysmap/forms/',
+	'class.selement.js' => 'pages/sysmap/',
+	'class.shape.js' => 'pages/sysmap/',
+	'class.link.js' => 'pages/sysmap/',
+	'class.cmap.js' => 'pages/sysmap/',
+	'sysmap.js' => 'pages/sysmap/',
 	'items.js' => 'pages/',
 	'report4.js' => 'pages/',
-	'setup.js' => 'pages/'
+	'setup.js' => 'pages/',
+	// fields
+	'class.field.js' => 'fields/',
+	'class.field.array.js' => 'fields/',
+	'class.field.checkbox.js' => 'fields/',
+	'class.field.hidden.js' => 'fields/',
+	'class.field.multiline.js' => 'fields/',
+	'class.field.multiselect.js' => 'fields/',
+	'class.field.radiolist.js' => 'fields/',
+	'class.field.set.js' => 'fields/',
+	'class.field.textarea.js' => 'fields/',
+	'class.field.textbox.js' => 'fields/',
+	'class.field.zselect.js' => 'fields/'
 ];
 
 $translate_strings = [
@@ -143,6 +196,7 @@ $translate_strings = [
 		'Cannot paste inaccessible widget.' => _('Cannot paste inaccessible widget.'),
 		'Copy' => _('Copy'),
 		'Delete' => _('Delete'),
+		'Editing other objects is not allowed in dashboard editing mode.' => _('Editing other objects is not allowed in dashboard editing mode.'),
 		'Failed to paste dashboard page.' => _('Failed to paste dashboard page.'),
 		'Failed to paste widget.' => _('Failed to paste widget.'),
 		'Failed to update dashboard page properties.' => _('Failed to update dashboard page properties.'),
@@ -203,6 +257,15 @@ $translate_strings = [
 		'No compatible widgets.' => _('No compatible widgets.'),
 		'Widget' => _('Widget')
 	],
+	'class.widget-edit.dialogue.js' => [
+		'Widget configuration will be reverted.' => _('Widget configuration will be reverted.')
+	],
+	'class.widget-edit.sandbox.js' => [
+		'Cannot add widget: not enough free space on the dashboard.' => _('Cannot add widget: not enough free space on the dashboard.')
+	],
+	'class.widget-edit.validator.js' => [
+		'Failed to update widget properties.' => _('Failed to update widget properties.')
+	],
 	'class.widget-field.multiselect.js' => [
 		'Dashboard' => _('Dashboard'),
 		'Widget' => _('Widget'),
@@ -258,42 +321,67 @@ $translate_strings = [
 		'S_SATURDAY_SHORT_BIG' => _x('S', 'Saturday short'),
 		'S_SUNDAY_SHORT_BIG' => _x('S', 'Sunday short')
 	],
-	'class.cmap.js' => [
-		'S_ON' => _('On'),
-		'S_OFF' => _('Off'),
-		'S_HIDDEN' => _('Hidden'),
-		'S_SHOWN' => _('Shown'),
+	'class.linkform.js' => [
+		'S_CANNOT_BE_EMPTY' => _('cannot be empty'),
+		'S_COLOR_OK' => _('Color (OK)'),
+		'S_EXPECTING_COLOR_CODE' => _('expecting hexadecimal color code (6 symbols)'),
 		'S_HOST' => _('Host'),
-		'S_MAP' => _('Map'),
-		'S_TRIGGER' => _('Trigger'),
 		'S_HOST_GROUP' => _('Host group'),
 		'S_IMAGE' => _('Image'),
+		'S_INCORRECT_ITEM_VALUE_TYPE' =>  _('incorrect item value type'),
+		'S_INCORRECT_VALUE' => _('Incorrect value for field "%1$s": %2$s.'),
+		'S_INDICATORS' => _('Indicators'),
+		'S_INVALID_PARAMETER' => _('Invalid parameter "%1$s": %2$s.'),
+		'S_ITEM' => _('Item'),
+		'S_LINK_HIGHLIGHT_IS_REQUIRED' => _('at least 1 link highlight is required'),
+		'S_LINK_THRESHOLD_IS_REQUIRED' => _('at least 1 link threshold is required'),
+		'S_LINK_TRIGGER_IS_REQUIRED' => _('at least 1 link trigger is required'),
+		'S_MAP' => _('Map'),
+		'S_TRIGGER' => _('Trigger')
+	],
+	'class.massform.js' => [
 		'S_DEFAULT' => _('Default'),
-		'S_PLEASE_SELECT_TWO_ELEMENTS' => _('Please select two elements'),
-		'S_TWO_MAP_ELEMENTS_SHOULD_BE_SELECTED' => _('Two map elements should be selected'),
-		'S_DELETE_SELECTED_ELEMENTS_Q' => _('Delete selected elements?'),
-		'S_DELETE_SELECTED_SHAPES_Q' => _('Delete selected shapes?'),
-		'S_BRING_TO_FRONT' => _('Bring to front'),
-		'S_BRING_FORWARD' => _('Bring forward'),
-		'S_SEND_BACKWARD' => _('Send backward'),
-		'S_SEND_TO_BACK' => _('Send to back'),
-		'S_REMOVE' => _('Remove'),
-		'S_NEW_ELEMENT' => _('New element'),
-		'S_COPY' => _('Copy'),
-		'S_PASTE' => _('Paste'),
-		'S_PASTE_SIMPLE' => _('Paste without external links'),
-		'S_INCORRECT_ELEMENT_MAP_LINK' => _('All links should have "Name" and "URL" specified'),
+		'S_HOST' => _('Host'),
+		'S_HOST_GROUP' => _('Host group'),
+		'S_IMAGE' => _('Image'),
+		'S_MAP' => _('Map'),
+		'S_TRIGGER' => _('Trigger')
+	],
+	'class.selementform.js' => [
+		'Host is not selected.' => _('Host is not selected.'),
+		'Host group is not selected.' => _('Host group is not selected.'),
+		'Map is not selected.' => _('Map is not selected.'),
 		'S_EACH_URL_SHOULD_HAVE_UNIQUE' =>
 			_('Each URL should have a unique name. Please make sure there is only one URL named'),
+		'S_DEFAULT' => _('Default'),
+		'S_INCORRECT_ELEMENT_MAP_LINK' => _('All links should have "Name" and "URL" specified'),
+		'Trigger is not selected.' => _('Trigger is not selected.')
+	],
+	'class.selement.js' => [
+		'S_IMAGE' => _('Image'),
+		'S_NEW_ELEMENT' => _('New element')
+	],
+	'class.cmap.js' => [
+		'S_BRING_FORWARD' => _('Bring forward'),
+		'S_BRING_TO_FRONT' => _('Bring to front'),
+		'S_COPY' => _('Copy'),
 		'S_DELETE_LINKS_BETWEEN_SELECTED_ELEMENTS_Q' => _('Delete links between selected elements?'),
+		'S_DELETE_SELECTED_ELEMENTS_Q' => _('Delete selected elements?'),
+		'S_DELETE_SELECTED_SHAPES_Q' => _('Delete selected shapes?'),
+		'S_HIDDEN' => _('Hidden'),
 		'S_MACRO_EXPAND_ERROR' => _('Cannot expand macros.'),
 		'S_NO_IMAGES' =>
 			_('You need to have at least one image uploaded to create map element. Images can be uploaded in Administration->General->Images section.'),
-		'S_COLOR_IS_NOT_CORRECT' => _('Color "%1$s" is not correct: expecting hexadecimal color code (6 symbols).'),
-		'Host is not selected.' => _('Host is not selected.'),
-		'Map is not selected.' => _('Map is not selected.'),
-		'Trigger is not selected.' => _('Trigger is not selected.'),
-		'Host group is not selected.' => _('Host group is not selected.')
+		'S_OFF' => _('Off'),
+		'S_ON' => _('On'),
+		'S_PASTE' => _('Paste'),
+		'S_PASTE_SIMPLE' => _('Paste without external links'),
+		'S_PLEASE_SELECT_TWO_ELEMENTS' => _('Please select two elements'),
+		'S_REMOVE' => _('Remove'),
+		'S_SEND_BACKWARD' => _('Send backward'),
+		'S_SEND_TO_BACK' => _('Send to back'),
+		'S_SHOWN' => _('Shown'),
+		'S_TWO_MAP_ELEMENTS_SHOULD_BE_SELECTED' => _('Two map elements should be selected')
 	],
 	'class.notification.js' => [
 		'S_PROBLEM_ON' => _('Problem on'),
@@ -324,6 +412,33 @@ $translate_strings = [
 		'S_COLLAPSE' => _('Collapse'),
 		'S_EXPAND' => _('Expand')
 	],
+	'class.form.js' => [
+		'Page received incorrect data' => _('Page received incorrect data')
+	],
+	'class.form.validator.js' => [
+		'An array is expected.' => _('An array is expected.'),
+		'Required field is missing.' => _('Required field is missing.'),
+		'This value does not match pattern.' => _('This value does not match pattern.'),
+		'This value is not a valid identifier.' => _('This value is not a valid identifier.'),
+		'This value is not a valid integer.' => _('This value is not a valid integer.'),
+		'This value is not a valid floating-point value.' => _('This value is not a valid floating-point value.'),
+		'This object already exists.' => _('This object already exists.'),
+		'This value is not a valid string.' => _('This value is not a valid string.'),
+		'This field cannot be empty.' => _('This field cannot be empty.'),
+		'This value is too long.' => _('This value is too long.'),
+		'This value cannot be one of %1$s.' => _('This value cannot be one of %1$s.'),
+		'This value cannot be %1$s.' => _('This value cannot be %1$s.'),
+		'This value must be %1$s.' => _('This value must be %1$s.'),
+		'This value must be no less than "%1$s".' => _('This value must be no less than "%1$s".'),
+		'This value must be no greater than "%1$s".' => _('This value must be no greater than "%1$s".'),
+		'This value must be one of %1$s.' => _('This value must be one of %1$s.'),
+		'Entry "%1$s" is not unique.' => _('Entry "%1$s" is not unique.'),
+		'within range %1$s' => _('within range %1$s'),
+		'within ranges %1$s' => _('within ranges %1$s'),
+		'one of %1$s' => _('one of %1$s'),
+		' or ' => _(' or '),
+		'%1$s: %2$s' => _('%1$s: %2$s')
+	],
 	'main.js' => [
 		'S_EXPAND' => _('Expand'),
 		'S_COLLAPSE' => _('Collapse'),
@@ -394,6 +509,7 @@ $translate_strings = [
 		'Graphs' => _('Graphs'),
 		'History' => _('History'),
 		'Host' => _('Host'),
+		'Host Wizard' => _('Host Wizard'),
 		'Inventory' => _('Inventory'),
 		'Latest data' => _('Latest data'),
 		'Latest values' => _('Latest values'),
@@ -432,11 +548,6 @@ $translate_strings = [
 		'No interface found' => _('No interface found'),
 		'Item type does not use interface' => _('Item type does not use interface')
 	],
-	'colorpicker.js' => [
-		'D' => _x('D', 'Default color option'),
-		'S_CLOSE' => _('Close'),
-		'Use default' => _('Use default')
-	],
 	'class.csvggraph.js' => [
 		'S_DISPLAYING_FOUND' => _('Displaying %1$s of %2$s found'),
 		'S_MINUTE_SHORT' => _x('m', 'minute short'),
@@ -470,6 +581,16 @@ $translate_strings = [
 	'class.navigationtree.js' => [
 		'Maintenance with data collection' => _('Maintenance with data collection'),
 		'Maintenance without data collection' => _('Maintenance without data collection')
+	],
+	'component.z-color-picker.js' => [
+		'Apply' => _('Apply'),
+		'Clear' => _('Clear'),
+		'D' => _x('D', 'Default color option'),
+		'No color' => _('No color'),
+		'Palette' => _('Palette'),
+		'Palette %1$d' => _('Palette %1$d'),
+		'Solid color' => _('Solid color'),
+		'Use default' => _('Use default')
 	]
 ];
 
@@ -482,9 +603,10 @@ if (empty($_GET['files'])) {
 		'main.js',
 		'common.js',
 		'component.z-bar-gauge.js',
-		'component.z-vertical.js',
+		'component.z-color-picker.js',
 		'component.z-select.js',
 		'component.z-sparkline.js',
+		'component.z-vertical.js',
 		'class.event-hub.js',
 		'class.event-hub.event.js',
 		'class.base-component.js',
@@ -496,6 +618,19 @@ if (empty($_GET['files'])) {
 		'class.cdate.js',
 		'class.cookie.js',
 		'class.curl.js',
+		'class.field.js',
+		'class.field.array.js',
+		'class.field.checkbox.js',
+		'class.field.hidden.js',
+		'class.field.multiline.js',
+		'class.field.multiselect.js',
+		'class.field.radiolist.js',
+		'class.field.set.js',
+		'class.field.textarea.js',
+		'class.field.textbox.js',
+		'class.field.zselect.js',
+		'class.form.js',
+		'class.form.validator.js',
 		'class.menu.js',
 		'class.menu-item.js',
 		'class.rpc.js',
